!*******************************************************************************
! Fortran module interfacing the GD Graphics Library http://www.boutell.com/gd/
! Version 1.0
! For documentation and all other relevant information:
! Mart.Rentmeester@nn-online.org    http://nn-online.org/code/
!*******************************************************************************

module gd

integer, parameter :: rp = kind(1d0)

!*** cptr should be have the same size as a c pointer
!*** It doesn't matter whether it is an integer or a real
integer, parameter :: cptr = kind(5)

integer, public, parameter :: gd_max_colors = 256

type gd_image
    private
    integer        :: width = 0
    integer        :: height = 0
    integer(cptr)  :: ptr = 0
    logical        :: fmode = .false.
end type

type gd_font
    private
    integer(cptr)  :: ptr = 0
end type

contains

!*******************************************************************************
subroutine gd_create_img(width,height,image,status)

implicit none
integer, intent(in)             :: width,height
type(gd_image), intent(out)     :: image
integer, intent(out), optional  :: status


if (present(status)) then
    call gd_create_palette_img(width,height,image,status)
else
    call gd_create_palette_img(width,height,image)
endif

end subroutine gd_create_img
!*******************************************************************************
subroutine gd_create_palette_img(width,height,image,status)

implicit none
integer, intent(in)             :: width,height
type(gd_image), intent(out)     :: image
integer, intent(out), optional  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

if (width <= 0 .or. height <= 0) then
    if (present(status)) status = 2
    return
endif

call cgd_image_create(width,height,image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        image%width = gd_c_width(image)
        image%height = gd_c_height(image)
        status = 0
    else
        status = 1
    endif
endif

end subroutine gd_create_palette_img
!*******************************************************************************
subroutine gd_create_truecolor_img(width,height,image,status)

implicit none
integer, intent(in)             :: width,height
type(gd_image), intent(out)     :: image
integer, intent(out), optional  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

if (width <= 0 .or. height <= 0) then
    if (present(status)) status = 2
    return
endif

call cgd_image_create_truecolor(width,height,image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        status = 0
        image%width = gd_c_width(image)
        image%height = gd_c_height(image)
    else
        status = 1
    endif
endif

end subroutine gd_create_truecolor_img
!*******************************************************************************
subroutine gd_destroy_img(image)

implicit none
type(gd_image), intent(inout)  :: image


call cgd_image_destroy(image%ptr)
image%ptr = 0
image%width = 0
image%height = 0
image%fmode = .false.

end subroutine gd_destroy_img
!*******************************************************************************
subroutine gd_allocate_color(image,r,g,b,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b
integer, intent(out)        :: color


call cgd_image_color_allocate(image%ptr,r,g,b,color)

end subroutine gd_allocate_color
!*******************************************************************************
subroutine gd_allocate_alpha_color(image,r,g,b,a,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b,a
integer, intent(out)        :: color


call cgd_image_color_allocate_alpha(image%ptr,r,g,b,a,color)

end subroutine gd_allocate_alpha_color
!*******************************************************************************
subroutine gd_deallocate_color(image,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color


call cgd_image_color_deallocate(image%ptr,color)

end subroutine gd_deallocate_color
!*******************************************************************************
subroutine gd_set_pixel(image,x,y,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y,color


call cgd_image_set_pixel(image%ptr,xm(image,x),ym(image,y),color)

end subroutine gd_set_pixel
!*******************************************************************************
subroutine gd_draw_line(image,x1,y1,x2,y2,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x1,y1,x2,y2,color


call cgd_image_line(image%ptr,xm(image,x1),ym(image,y1),  &
                              xm(image,x2),ym(image,y2),color)

end subroutine gd_draw_line
!*******************************************************************************
subroutine gd_draw_rectangle(image,x1,y1,x2,y2,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x1,y1,x2,y2,color


call cgd_image_rectangle(image%ptr,xm(image,x1),ym(image,y1),  &
                                   xm(image,x2),ym(image,y2),color)

end subroutine gd_draw_rectangle
!*******************************************************************************
subroutine gd_draw_polygon(image,n,x,y,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: n,x(n),y(n),color


call cgd_image_polygon(image%ptr,n,xm(image,x),ym(image,y),color)

end subroutine gd_draw_polygon
!*******************************************************************************
subroutine gd_draw_arc(image,x,y,w,h,s,e,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y,w,h,s,e,color


call cgd_image_arc(image%ptr,xm(image,x),ym(image,y),w,h,s,e,color)

end subroutine gd_draw_arc
!*******************************************************************************
subroutine gd_draw_filled_polygon(image,n,x,y,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: n,x(n),y(n),color


call cgd_image_filled_polygon(image%ptr,n,xm(image,x),ym(image,y),color)

end subroutine gd_draw_filled_polygon
!*******************************************************************************
subroutine gd_draw_filled_rectangle(image,x1,y1,x2,y2,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x1,y1,x2,y2,color


call cgd_image_filled_rectangle(image%ptr,xm(image,x1),ym(image,y1),  &
                                          xm(image,x2),ym(image,y2),color)

end subroutine gd_draw_filled_rectangle
!*******************************************************************************
subroutine gd_draw_filled_arc(image,x,y,w,h,s,e,color,style)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y,w,h,s,e,color,style


call cgd_image_filled_arc(image%ptr,xm(image,x),ym(image,y),w,h,s,e,color,style)

end subroutine gd_draw_filled_arc
!*******************************************************************************
subroutine gd_draw_filled_ellipse(image,x,y,w,h,color,style)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y,w,h,color,style


call cgd_image_filled_ellipse(image%ptr,xm(image,x),ym(image,y),w,h,color,style)

end subroutine gd_draw_filled_ellipse
!*******************************************************************************
subroutine gd_write_img_as_png(img,file)

implicit none
type(gd_image), intent(in)  :: img
character(*), intent(in)    :: file


call cgd_image_png(img%ptr,trim(file)//char(0))

end subroutine gd_write_img_as_png
!*******************************************************************************
#ifndef WITHOUT_GIF
subroutine gd_create_img_from_gif(file,image,status)

implicit none
character(*), intent(in)        :: file
type(gd_image), intent(out)     :: image
integer, optional, intent(out)  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

call cgd_image_create_from_gif(trim(file)//char(0),image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        status = 0
    else
        status = 1
    endif
endif
image%width = gd_c_width(image)
image%height = gd_c_height(image)

end subroutine gd_create_img_from_gif
#endif
!*******************************************************************************
subroutine gd_create_img_from_png(file,image,status)

implicit none
character(*), intent(in)        :: file
type(gd_image), intent(out)     :: image
integer, optional, intent(out)  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

call cgd_image_create_from_png(trim(file)//char(0),image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        status = 0
    else
        status = 1
    endif
endif
image%width = gd_c_width(image)
image%height = gd_c_height(image)

end subroutine gd_create_img_from_png
!*******************************************************************************
#ifndef WITHOUT_GIF
subroutine gd_write_img_as_gif(img,file)

implicit none
type(gd_image), intent(in)  :: img
character(*), intent(in)    :: file


call cgd_image_gif(img%ptr,trim(file)//char(0))

end subroutine gd_write_img_as_gif
#endif
!*******************************************************************************
subroutine gd_write_img_as_gd(image,file)

implicit none
type(gd_image), intent(in)  :: image
character(*), intent(in)    :: file


call cgd_image_gd(image%ptr,trim(file)//char(0))

end subroutine gd_write_img_as_gd
!*******************************************************************************
subroutine gd_create_img_from_gd(file,image,status)

implicit none
character(*), intent(in)        :: file
type(gd_image), intent(out)     :: image
integer, intent(out), optional  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

call cgd_image_create_from_gd(trim(file)//char(0),image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        status = 0
    else
        status = 1
    endif
endif
image%width = gd_c_width(image)
image%height = gd_c_height(image)

end subroutine gd_create_img_from_gd
!*******************************************************************************
#ifndef WITHOUT_JPEG
subroutine gd_create_img_from_jpeg(file,image,status)

implicit none
character(*), intent(in)        :: file
type(gd_image), intent(out)     :: image
integer, intent(out), optional  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

call cgd_image_create_from_jpeg(trim(file)//char(0),image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        status = 0
    else
        status = 1
    endif
endif
image%width = gd_c_width(image)
image%height = gd_c_height(image)

end subroutine gd_create_img_from_jpeg
!*******************************************************************************
subroutine gd_write_img_as_jpeg(img,file,quality)

implicit none
type(gd_image), intent(in)  :: img
character(*), intent(in)    :: file
integer, intent(in)         :: quality


call cgd_image_jpeg(img%ptr,trim(file)//achar(0),quality)

end subroutine gd_write_img_as_jpeg
#endif
!*******************************************************************************
subroutine gd_draw_char(image,font,x,y,c,color)

implicit none
type(gd_image), intent(in)  :: image
type(gd_font), intent(in)   :: font
integer, intent(in)         :: x,y,color
character(1), intent(in)    :: c
integer                     :: xx,yy


if (image%fmode) then
    xx = xm(image,x)
    yy = ym(image,y) + gd_font_width(font)
else
    xx = x
    yy = y
endif
call cgd_image_char(image%ptr,font%ptr,xx,yy,iachar(c),color)

end subroutine gd_draw_char
!*******************************************************************************
subroutine gd_draw_char_up(image,font,x,y,c,color)

implicit none
type(gd_image), intent(in)  :: image
type(gd_font), intent(in)   :: font
integer, intent(in)         :: x,y,color
character(1), intent(in)    :: c
integer                     :: xx,yy


if (image%fmode) then
    xx = xm(image,x) + gd_font_width(font)
    yy = ym(image,y)
else
    xx = x
    yy = y
endif
call cgd_image_char_up(image%ptr,font%ptr,xx,yy,iachar(c),color)

end subroutine gd_draw_char_up
!*******************************************************************************
subroutine gd_draw_string(image,font,x,y,string,color)

implicit none
type(gd_image), intent(in)  :: image
type(gd_font), intent(in)   :: font
integer, intent(in)         :: x,y,color
character(*), intent(in)    :: string
integer                     :: xx,yy


if (image%fmode) then
    xx = xm(image,x)
    yy = ym(image,y) + gd_font_width(font)
else
    xx = x
    yy = y
endif
call cgd_image_string(image%ptr,font%ptr,xx,yy,trim(string)//achar(0),color)

end subroutine gd_draw_string
!*******************************************************************************
subroutine gd_draw_string_up(image,font,x,y,string,color)

implicit none
type(gd_image), intent(in)  :: image
type(gd_font), intent(in)   :: font
integer, intent(in)         :: x,y,color
character(*), intent(in)    :: string
integer                     :: xx,yy


if (image%fmode) then
    xx = xm(image,x) + gd_font_width(font)
    yy = ym(image,y)
else
    xx = x
    yy = y
endif
call cgd_image_string_up(image%ptr,font%ptr,xx,yy,trim(string)//achar(0),color)

end subroutine gd_draw_string_up
!*******************************************************************************
subroutine gd_set_line_thickness(image,thickness)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: thickness


call cgd_image_set_line_thickness(image%ptr,thickness)

end subroutine gd_set_line_thickness
!*******************************************************************************
function gd_font_tiny()

implicit none
type(gd_font)  :: gd_font_tiny


call cgd_font_tiny(gd_font_tiny%ptr)

end function gd_font_tiny
!*******************************************************************************
function gd_font_small()

implicit none
type(gd_font)  :: gd_font_small


call cgd_font_small(gd_font_small%ptr)

end function gd_font_small
!*******************************************************************************
function gd_font_medium_bold()

implicit none
type(gd_font)  :: gd_font_medium_bold


call cgd_font_medium_bold(gd_font_medium_bold%ptr)

end function gd_font_medium_bold
!*******************************************************************************
function gd_font_large()

implicit none
type(gd_font)  :: gd_font_large


call cgd_font_large(gd_font_large%ptr)

end function gd_font_large
!*******************************************************************************
function gd_font_giant()

implicit none
type(gd_font)  :: gd_font_giant


call cgd_font_giant(gd_font_giant%ptr)

end function gd_font_giant
!*******************************************************************************
#ifndef WITHOUT_FT
subroutine gd_draw_ft_string(image,color,font,ptsize,angle,x,y,string,status)

implicit none
type(gd_image), intent(in)      :: image
character(*), intent(in)        :: font,string
real(rp), intent(in)            :: ptsize,angle
integer, intent(in)             :: color,x,y
integer, intent(out), optional  :: status
integer                         :: b(8),s


call cgd_image_string_ft(image%ptr,b,color,font//achar(0),ptsize,angle,  &
                          xm(image,x),ym(image,y),string//achar(0),s)
if (present(status)) status = s

end subroutine gd_draw_ft_string
!*******************************************************************************
subroutine gd_ft_string_size(font,ptsize,string,width,height)

implicit none
character(*), intent(in)  :: font,string
real(rp), intent(in)      :: ptsize
integer, intent(out)      :: width,height
type(gd_image)            :: image
integer                   :: b(8),status


call cgd_image_string_ft(image%ptr,b,0,font//achar(0),ptsize,0.0_rp,  &
                          0,0,string//achar(0),status)
width  = abs(b(5))
height = abs(b(6))

end subroutine gd_ft_string_size
#endif
!*******************************************************************************
subroutine gd_fill(image,x,y,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y,color


call cgd_image_fill(image%ptr,xm(image,x),ym(image,y),color)

end subroutine gd_fill
!*******************************************************************************
subroutine gd_fill_to_border(image,x,y,bordercolor,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y,bordercolor,color


call cgd_image_fill_to_border(image%ptr,xm(image,x),ym(image,y),  &
                              bordercolor,color)

end subroutine gd_fill_to_border
!*******************************************************************************
function gd_c_width(image)

implicit none
type(gd_image), intent(in)  :: image
integer                     :: gd_c_width


call cgd_width(image%ptr,gd_c_width)

end function gd_c_width
!*******************************************************************************
function gd_c_height(image)

implicit none
type(gd_image), intent(in)  :: image
integer                     :: gd_c_height


call cgd_height(image%ptr,gd_c_height)

end function gd_c_height
!*******************************************************************************
function gd_width(image)

implicit none
type(gd_image), intent(in)  :: image
integer                     :: gd_width


gd_width = image%width

end function gd_width
!*******************************************************************************
function gd_height(image)

implicit none
type(gd_image), intent(in)  :: image
integer                     :: gd_height


gd_height = image%height

end function gd_height
!*******************************************************************************
subroutine gd_create_img_from_xbm(file,image,status)

implicit none
character(*), intent(in)        :: file
type(gd_image), intent(out)     :: image
integer, optional, intent(out)  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

call cgd_image_create_from_xbm(trim(file)//char(0),image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        status = 0
    else
        status = 1
    endif
endif
image%width = gd_c_width(image)
image%height = gd_c_height(image)

end subroutine gd_create_img_from_xbm
!*******************************************************************************
#ifndef WITHOUT_XPM
subroutine gd_create_img_from_xpm(file,image,status)

implicit none
character(*), intent(in)        :: file
type(gd_image), intent(out)     :: image
integer, optional, intent(out)  :: status


if (image%ptr /= 0) call gd_destroy_img(image)

call cgd_image_create_from_xpm(trim(file)//char(0),image%ptr)
if (present(status)) then
    if (image%ptr /= 0) then
        status = 0
    else
        status = 1
    endif
endif
image%width = gd_c_width(image)
image%height = gd_c_height(image)

end subroutine gd_create_img_from_xpm
#endif
!*******************************************************************************
subroutine gd_set_brush(image,brush)

implicit none
type(gd_image), intent(in)  :: image,brush


call cgd_image_set_brush(image%ptr,brush%ptr)

end subroutine gd_set_brush
!*******************************************************************************
subroutine gd_set_tile(image,tile)

implicit none
type(gd_image), intent(in)  :: image,tile


call cgd_image_set_tile(image%ptr,tile%ptr)

end subroutine gd_set_tile
!*******************************************************************************
subroutine gd_set_style(image,style,length)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: length,style(length)


call cgd_image_set_style(image%ptr,style,length)

end subroutine gd_set_style
!*******************************************************************************
subroutine gd_set_alpha_blending(image,b)

implicit none
type(gd_image), intent(in)  :: image
logical, intent(in)         :: b
integer                     :: i


i = 1
if (b) i = 0
call cgd_image_set_alpha_blending(image%ptr,i)

end subroutine gd_set_alpha_blending
!*******************************************************************************
subroutine gd_save_alpha(image,b)

implicit none
type(gd_image), intent(in)  :: image
logical, intent(in)         :: b
integer                     :: i


i = 0
if (b) i = 1
call cgd_image_save_alpha(image%ptr,i)

end subroutine gd_save_alpha
!*******************************************************************************
function gd_blue(image,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color
integer                     :: gd_blue


call cgd_blue(image%ptr,color,gd_blue)

end function gd_blue
!*******************************************************************************
function gd_alpha(image,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color
integer                     :: gd_alpha


call cgd_blue(image%ptr,color,gd_alpha)

end function gd_alpha
!*******************************************************************************
function gd_pixel_color(image,x,y)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y
integer                     :: gd_pixel_color


call cgd_pixel_color(image,xm(image,x),ym(image,y),gd_pixel_color)

end function gd_pixel_color
!*******************************************************************************
function gd_green(image,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color
integer                     :: gd_green


call cgd_green(image%ptr,color,gd_green)

end function gd_green
!*******************************************************************************
function gd_red(image,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color
integer                     :: gd_red


call cgd_red(image%ptr,color,gd_red)

end function gd_red
!*******************************************************************************
function gd_within_bounds(image,x,y)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x,y
logical                     :: gd_within_bounds
integer                     :: safe


call cgd_image_bounds_safe(image,xm(image,x),ym(image,y),safe)
gd_within_bounds = (safe /= 0)

end function gd_within_bounds
!*******************************************************************************
function gd_closest_color(image,r,g,b)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b
integer                     :: gd_closest_color


call cgd_image_color_closest(image%ptr,r,g,b,gd_closest_color)

end function gd_closest_color
!*******************************************************************************
function gd_hwb_closest_color(image,r,g,b)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b
integer                     :: gd_hwb_closest_color


call cgd_image_color_closest_hwb(image%ptr,r,g,b,gd_hwb_closest_color)

end function gd_hwb_closest_color
!*******************************************************************************
function gd_closest_alpha_color(image,r,g,b,a)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b,a
integer                     :: gd_closest_alpha_color


call cgd_image_color_closest_alpha(image%ptr,r,g,b,a,gd_closest_alpha_color)

end function gd_closest_alpha_color
!*******************************************************************************
function gd_exact_color(image,r,g,b)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b
integer                     :: gd_exact_color


call cgd_image_color_exact(image%ptr,r,g,b,gd_exact_color)

end function gd_exact_color
!*******************************************************************************
function gd_resolve_alpha_color(image,r,g,b,a)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b,a
integer                     :: gd_resolve_alpha_color


call cgd_image_color_resolve_alpha(image%ptr,r,g,b,a,gd_resolve_alpha_color)

end function gd_resolve_alpha_color
!*******************************************************************************
function gd_resolve_color(image,r,g,b)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: r,g,b
integer                     :: gd_resolve_color


call cgd_image_color_resolve(image%ptr,r,g,b,gd_resolve_color)

end function gd_resolve_color
!*******************************************************************************
function gd_interlace(image)

implicit none
type(gd_image), intent(in)  :: image
logical                     :: gd_interlace
integer                     :: i


call cgd_image_get_interlaced(image%ptr,i)
gd_interlace = (i /= 0)

end function gd_interlace
!*******************************************************************************
function gd_transparent_color(image)

implicit none
type(gd_image), intent(in)  :: image
integer                     :: gd_transparent_color


call cgd_image_get_transparent(image%ptr,gd_transparent_color)

end function gd_transparent_color
!*******************************************************************************
subroutine gd_set_transparent_color(image,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color


call cgd_image_transparent(image%ptr,color)

end subroutine gd_set_transparent_color
!*******************************************************************************
subroutine gd_set_interlace(image,interlace)

implicit none
type(gd_image), intent(in)  :: image
logical, intent(in)         :: interlace
integer                     :: i = 0


if (interlace) i = 1
call cgd_image_interlace(image%ptr,i)

end subroutine gd_set_interlace
!*******************************************************************************
function gd_number_of_colors(image)

implicit none
type(gd_image), intent(in)  :: image
integer                     :: gd_number_of_colors


call cgd_image_colors_total(image%ptr,gd_number_of_colors)

end function gd_number_of_colors
!*******************************************************************************
function gd_brushed()

implicit none
integer  :: gd_brushed


call cgd_brushed(gd_brushed)

end function gd_brushed
!*******************************************************************************
function gd_styled()

implicit none
integer  :: gd_styled


call cgd_styled(gd_styled)

end function gd_styled
!*******************************************************************************
function gd_styled_brushed()

implicit none
integer  :: gd_styled_brushed


call cgd_styled_brushed(gd_styled_brushed)

end function gd_styled_brushed
!*******************************************************************************
function gd_tiled()

implicit none
integer  :: gd_tiled


call cgd_tiled(gd_tiled)

end function gd_tiled
!*******************************************************************************
function gd_pie()

implicit none
integer  :: gd_pie


call cgd_pie(gd_pie)

end function gd_pie
!*******************************************************************************
function gd_chord()

implicit none
integer  :: gd_chord


call cgd_chord(gd_chord)

end function gd_chord
!*******************************************************************************
function gd_no_fill()

implicit none
integer  :: gd_no_fill


call cgd_nofill(gd_no_fill)

end function gd_no_fill
!*******************************************************************************
function gd_edged()

implicit none
integer  :: gd_edged


call cgd_edged(gd_edged)

end function gd_edged
!*******************************************************************************
function gd_transparent()

implicit none
integer  :: gd_transparent


call cgd_transparent(gd_transparent)

end function gd_transparent
!*******************************************************************************
function gd_anti_aliased()

implicit none
integer  :: gd_anti_aliased


call cgd_anti_aliased(gd_anti_aliased)

end function gd_anti_aliased
!*******************************************************************************
subroutine gd_set_clip_rectangle(image,x1,y1,x2,y2)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x1,y1,x2,y2
integer                     :: xx1,yy1,xx2,yy2


if (image%fmode) then
    xx1 = xm(image,x1)
    yy1 = ym(image,y2)
    xx2 = xm(image,x2)
    yy2 = ym(image,y1)
else
    xx1 = x1
    yy1 = y1
    xx2 = x2
    yy2 = y2
endif
call cgd_image_set_clip(image%ptr,xx1,yy1,xx2,yy2)

end subroutine gd_set_clip_rectangle
!*******************************************************************************
subroutine gd_get_clip_rectangle(image,x1,y1,x2,y2)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(out)        :: x1,y1,x2,y2
integer                     :: s


call cgd_image_get_clip(image%ptr,x1,y1,x2,y2)
if (image%fmode) then
    x1 = xm(image,x1)
    s  = xm(image,y2)
    x2 = xm(image,x2)
    y2 = xm(image,y1)
    y1 = s
endif

end subroutine gd_get_clip_rectangle
!*******************************************************************************
subroutine gd_set_anti_aliased(image,color)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color


call cgd_image_set_aa(image%ptr,color)

end subroutine gd_set_anti_aliased
!*******************************************************************************
subroutine gd_set_anti_aliased_dont_blend(image,color,bcolor)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: color,bcolor


call cgd_image_set_aa_nb(image%ptr,color,bcolor)

end subroutine gd_set_anti_aliased_dont_blend
!*******************************************************************************
subroutine gd_set_fmode(image,mode)

implicit none
type(gd_image), intent(inout)  :: image
logical, intent(in)            :: mode


image%fmode = mode

end subroutine gd_set_fmode
!*******************************************************************************
function gd_fmode(image)

implicit none
type(gd_image), intent(in)  :: image
logical                     :: gd_fmode


gd_fmode = image%fmode

end function gd_fmode
!*******************************************************************************
elemental function xm(image,x)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: x
integer                     :: xm


xm = x
if (image%fmode) xm = x + 1

end function xm
!*******************************************************************************
elemental function ym(image,y)

implicit none
type(gd_image), intent(in)  :: image
integer, intent(in)         :: y
integer                     :: ym


ym = y
if (image%fmode) ym = image%height - y

end function ym
!*******************************************************************************
function gd_font_width(f)

implicit none
type(gd_font), intent(in)  :: f
integer                    :: gd_font_width


call cgd_font_width(f%ptr,gd_font_width)

end function gd_font_width
!*******************************************************************************
function gd_font_height(f)

implicit none
type(gd_font), intent(in)  :: f
integer                    :: gd_font_height


call cgd_font_height(f%ptr,gd_font_height)

end function gd_font_height
!*******************************************************************************
end module gd
